# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2011 Domsense s.r.l. (<http://www.domsense.com>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields, osv
from tools.translate import _

class account_invoice(osv.osv):
    _inherit = "account.invoice"
    _columns = {
        'vat_on_cash': fields.boolean('Vat on cash receipts'),
        }
account_invoice()

class account_voucher(osv.osv):
    _inherit = "account.voucher"

    def action_move_line_create(self, cr, uid, ids, context=None):
        if isinstance(ids, (int, long)):
            ids = [ids]
        if not context:
            context={}
        res = super(account_voucher, self).action_move_line_create(cr, uid, ids, context)
        invoice_pool=self.pool.get('account.invoice')
        move_pool=self.pool.get('account.move')
        line_pool=self.pool.get('account.move.line')

        invoices = []
        # search for reconciled invoices and compute partial payment
        for voucher in self.browse(cr, uid, ids):
            for line in voucher.move_ids:
                if line.reconcile_partial_id:
                    paid_amount = 0.0
                    inv_amount = 0.0
                    for reconcile_line in line.reconcile_partial_id.line_partial_ids:
                        if reconcile_line.invoice:
                            invoice_id = reconcile_line.invoice.id
                            inv_amount = reconcile_line.invoice.amount_total
                        elif reconcile_line.credit > 0.0:
                            paid_amount += reconcile_line.credit
                    inv_tuple = (invoice_id, paid_amount / inv_amount)
                    if inv_tuple not in invoices:
                        invoices.append(inv_tuple)
                if line.reconcile_id:
                    for reconcile_line in line.reconcile_id.line_id:
                        if reconcile_line.invoice:
                            if (reconcile_line.invoice.id,1) not in invoices:
                                invoices.append((reconcile_line.invoice.id,1))
        
        for invoice_tuple in invoices:
            invoice = invoice_pool.browse(cr, uid, invoice_tuple[0], context)
            if invoice.vat_on_cash:
                move_data = {
                    'name': _('VAT on cash') + ' - ' + invoice.move_id.name,
                    }
                if invoice.journal_id.vat_on_cash_related_journal_id:
                    move_data['journal_id'
                        ] = invoice.journal_id.vat_on_cash_related_journal_id.id
                else:
                    move_data['journal_id'] = invoice.journal_id.id
                move_id = move_pool.create(cr, uid, move_data)
                move = move_pool.browse(cr, uid, move_id)

                for move_line in invoice.move_id.line_id:
                    line_count = 0
                    if move_line.credit > 0.0:
                        if move_line.account_id.vat_on_cash_related_account_id:
                            line_count +=1

                            line_data = {
                                'name': _('VAT on cash') + ' - ' + move_line.name,
                                'debit': move_line.debit * invoice_tuple[1],
                                'credit': move_line.credit * invoice_tuple[1],
                                'account_id': move_line.account_id.vat_on_cash_related_account_id.id,
                                'move_id': move_id,
                                'journal_id':  move.journal_id.id,
                                'period_id':  move.period_id.id,
                                }

                            shadow_line_data = {
                                'name': _('VAT on cash') + ' - ' + move_line.name,
                                'debit': move_line.credit * invoice_tuple[1],
                                'credit': move_line.debit * invoice_tuple[1],
                                'account_id': move_line.account_id.id,
                                'move_id': move_id,
                                'journal_id':  move.journal_id.id,
                                'period_id':  move.period_id.id,
                                }

                            if move_line.tax_code_id:
                                shadow_line_data['tax_code_id'] = move_line.tax_code_id.id
                                shadow_line_data['tax_amount'] = (shadow_line_data['credit']
                                    - shadow_line_data['debit'])
                                if move_line.tax_code_id.vat_on_cash_related_tax_code_id:
                                    line_data['tax_code_id'
                                        ] = move_line.tax_code_id.vat_on_cash_related_tax_code_id.id
                                    line_data['tax_amount'] = (line_data['credit']
                                        - line_data['debit'])

                            line_pool.create(cr, uid, line_data)
                            line_pool.create(cr, uid, shadow_line_data)
                    if line_count == 0:
                        raise osv.except_osv(_('Error!'), _('You are trying to reconcile a "Vat on cash receipts" invoice, but no account used by the invoice has a related account used for real registrations on a VAT on cash basis'))

                move_pool.post(cr, uid, [move_id], context={})

        return res

account_voucher()

class account_account(osv.osv):
    _inherit = "account.account"
    _columns = {
        'vat_on_cash_related_account_id': fields.many2one('account.account', 'Account for VAT on cash', help='Related account used for real registrations on a VAT on cash basis'),
        }
account_account()

class account_tax_code(osv.osv):
    _inherit = "account.tax.code"
    _columns = {
        'vat_on_cash_related_tax_code_id': fields.many2one('account.tax.code', 'Tax code for VAT on cash', help='Related tax code used for real registrations on a VAT on cash basis'),
        }
account_tax_code()

class account_journal(osv.osv):
    _inherit = "account.journal"
    _columns = {
        'vat_on_cash_related_journal_id': fields.many2one('account.journal', 'Journal for VAT on cash', help='Related journal used for real registrations on a VAT on cash basis'),
        }
account_journal()
